<?php
namespace Zeedhi\Tools\DatabaseReader;

use Doctrine\DBAL\Connection;
use Zeedhi\Tools\DTO;
use Zeedhi\Tools\DatabaseReader\Adapters\Oracle;
use Zeedhi\Tools\DatabaseReader\Adapters\MySQL;
use Zeedhi\Tools\DatabaseReader\Adapters\PostgreSQL;

class DatabaseReader {

    /** @var \Zeedhi\Tools\DatabaseReader\AdapterReader */
    protected $adapterReader;

    public function __construct(AdapterReader $adapterReader) {
        $this->adapterReader = $adapterReader;
    }

    public static function factoryFromConnection(Connection $connection, array $extras = array()) {
        $adapter = null;
        $connectionDriverName = $connection->getDriver()->getName();
        switch($connectionDriverName) {
            case 'oci8':
                $adapter = new Oracle($connection);
                if (isset($extras['sequenceNameProvider'])) {
                    $adapter->setSequenceNameProvider(new $extras['sequenceNameProvider']());
                }

                if (isset($extras['custom_owner'])) {
                    $adapter->setOwnerName($extras['custom_owner']);
                }
                break;
			case 'pdo_mysql':
				$adapter = new MySQL($connection);
				break;
			case 'pdo_pgsql':
				$adapter = new PostgreSQL($connection);
				break;
            default:
                throw Exception::unsupportedConnectionDriver($connectionDriverName);
                break;
        }

        return new self($adapter);
    }

    /**
     * Get name From all tables in database.
     *
     * @return string[]
     */
    public function getAllTableNames() {
        return $this->adapterReader->getAllTableNames();
    }

    /**
     * Get a table DTObject with all data about chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws Exception When table was not found in database.
     *
     * @return DTO\Table
     */
    public function getTable($tableName) {
        return $this->adapterReader->getTable($tableName);
    }

    /**
     * Get all columns From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws Exception When table was not found in database.
     *
     * @return DTO\Column[]
     */
    public function getColumnsFromTable($tableName) {
        return $this->adapterReader->getColumnsFromTable($tableName);
    }

    /**
     * Get primary key From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws Exception When table was not found in database.
     * @throws Exception When table has no primary key.
     *
     * @return DTO\Constraints\PrimaryKey
     */
    public function getPrimaryKeyFromTable($tableName) {
        return $this->adapterReader->getPrimaryKeyFromTable($tableName);
    }

    /**
     * Get foreign keys From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws Exception When table was not found in database.
     *
     * @return DTO\Constraints\ForeignKey[]
     */
    public function getForeignKeysFromTable($tableName) {
        return $this->adapterReader->getForeignKeysFromTable($tableName);
    }

    /**
     * Get unique constraints From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws Exception When table was not found in database.
     *
     * @return DTO\Constraints\Unique[]
     */
    public function getUniqueConstraintsFromTable($tableName) {
        return $this->adapterReader->getUniqueConstraintsFromTable($tableName);
    }

    public function getForeignKeyByNameAndTableName($constraintName, $tableName) {
        return $this->adapterReader->getForeignKeyByNameAndTableName($constraintName, $tableName);
    }

    /**
     * Get the sequence of that table.
     *
     * @param string $tableName  Name of table.
     * @param string $columnName Name of column that uses that sequence.
     *
     * @return DTO\Sequence
     */
    public function getSequenceFromTableAndColumn($tableName, $columnName) {
        return $this->adapterReader->getSequenceFromTableAndColumn($tableName, $columnName);
    }

    /**
     * Get DTO Check constraints for choosen table.
     *
     * @param string $tableName
     *
     * @return DTO\Constraints\Check[]
     */
    public function getCheckConstraint($tableName) {
        return $this->adapterReader->getCheckConstraints($tableName);
    }

    public function getDataFromRefConstraint($constraintName) {
        $row = $this->adapterReader->getDataFromRefConstraint($constraintName)[0];
        return $row;
    }
}